require 'json'
require 'rest-client'

module MapCoder

  private

  class EmptyAddressError < RuntimeError; end
  class UnknownGoogleAPIResponseError < RuntimeError; end

  REQUEST_URL = 'http://maps.google.com/maps/api/geocode/json'
  GOOGLE_API_SUCCESSFUL_RESPONSE = 'OK'
  GOOGLE_API_NOT_FOUND_RESPONSE = 'ZERO_RESULTS'
  GOOGLE_API_OVER_QUERY_LIMIT_RESPONSE = 'OVER_QUERY_LIMIT'

  public

  def self.coordinates(address)
    raise EmptyAddressError if address.nil? || address.empty?
    escaped_address = CGI.escape(address)
    url = "#{REQUEST_URL}?address=#{escaped_address}&sensor=false"
    attempt_number = 0

    loop do
      attempt_number += 1

      begin
        raw_response = RestClient.get(url)
      rescue
        return { status: :api_unavailable }
      end

      response = JSON.parse(raw_response)
      status = response['status']

      if status == GOOGLE_API_SUCCESSFUL_RESPONSE
        coordinates = response['results'].map { |result| result['geometry']['location'] }
        coordinates.map! { |result| { latitude: result['lat'], longitude: result['lng'] } }
        return { status: :ok, coordinates: coordinates }

      elsif status == GOOGLE_API_NOT_FOUND_RESPONSE
        return { status: :ok, coordinates: [] } 

      elsif status == GOOGLE_API_OVER_QUERY_LIMIT_RESPONSE
        if attempt_number == 1
          # Failed for the first time, sleep for 2 seconds and retry
          sleep 2
        else
          # Failed again, we are limited for the day
          return { status: :over_daily_limit }
        end
        
      else
        raise UnknownGoogleAPIResponseError
      end
    end
  end
end
